#include <stdio.h>
#include <string.h>
#include <gba.h>
#include <screen_mode.h>
#include <interrupt.h>
#include "basic_types_defs.h"
#include "game.h"
#include "automaton.h"
#include "ihm.h"
#include "bg_data.h"
#include "letters_data.h"
#include "popup_data.h"
#include "sfonts_data.h"
#include "init.h"
#include "printing.h"
#include "interrupts.h"
#include "key_input.h"
#include "gfx_effects.h"
#include "wlist_char_table.h"
#include "progress_data.h"
#include "intro.h"
#include "time.h"
#include "messages.h"
#include "game_over.h"
#include "fame.h"
#include "fame_data.h"
#include "wav_bad_key.h"
#include "wav_next_level.h"
#include "wav_inexist.h"
#include "wav_timer.h"



/* Effacement de la liste des propositions prcdentes. */
static void clear_submissions_wlist ()
{
  short line, column ;
  u16 *wlist_map = SCREEN_BASE_BLOCK_25_ADDR ;

  for (line = LIST1_AREA_Y_START ; line <= LIST1_AREA_Y_STOP; line++) {
    for (column = LIST1_AREA_X_START ; column <= LIST2_AREA_X_STOP; column++)
      wlist_map [(line * (256/8)) + column] =
	(u16) WLIST_SFONTS_START_INDEX | (WLIST_PALETTE_SLOT << 12) ;
  }

}




static void background_refresh (struct game *game)
{
  short i ;
  int sugg_len ;
  u16 *bg_map = SCREEN_BASE_BLOCK_24_ADDR ;
  u16 tile_info, tile_on_screen ;

  /* Affichage de la suggestion. */
  sugg_len = strlen (game->submission) ;
  for (i = 0; i < NB_LETTERS; i++) {
    if (i >= sugg_len)
      tile_info = LARGE_FONTS_INDEX ;  /* Lettre vide. */
    else
      tile_info = LARGE_FONTS_INDEX + ((1 + game->submission[i] - 'a') * 4) ;
    /* La position est dtermine par les coordonnes de la lettre. */
    tile_on_screen =
      (SUBMISSION_AREA_Y_START * (256 / 8)) /* Offset de ligne. */
	                                     /* 256 : Largeur map en pixels. */
	                                     /* 8 : Nbre pixels par tile. */
      + 
      (SUBMISSION_AREA_X_START + (i * 2)) ;      /* Offset de colonne. */
    /* Et finalement, on crit les 4 tiles composant la boule. */
    bg_map[tile_on_screen] = tile_info ;
    bg_map[tile_on_screen + 1] = tile_info + 1 ;
    tile_on_screen =
      tile_on_screen + (256 / 8) ;    /* 256 : Largeur map en pixels. */
	                                /* 8 : Nbre pixels par tile. */
    bg_map[tile_on_screen] = tile_info + 2 ;
    bg_map[tile_on_screen + 1] = tile_info + 3 ;
  }

  /* Affichage des lettres autorises. */
  for (i = 0; i < NB_LETTERS; i++) {
    if (i >= game->nb_letters_left)
      tile_info = LARGE_FONTS_INDEX ;
    else
      tile_info = LARGE_FONTS_INDEX + ((1 + game->letters[i] - 'a') * 4) ;
    /* La position est dtermine par les coordonnes de la lettre. */
    tile_on_screen =
      (AUTHORISED_AREA_Y_START * (256 / 8)) /* Offset de ligne. */
	                                     /* 256 : Largeur map en pixels. */
	                                     /* 8 : Nbre pixels par tile. */
      + 
      (AUTHORISED_AREA_X_START + (i * 2)) ;      /* Offset de colonne. */
    /* Et finalement, on crit les 4 tiles composant la boule. */
    bg_map[tile_on_screen] = tile_info ;
    bg_map[tile_on_screen + 1] = tile_info + 1 ;
    tile_on_screen =
      tile_on_screen + (256 / 8) ;    /* 256 : Largeur map en pixels. */
	                                /* 8 : Nbre pixels par tile. */
    bg_map[tile_on_screen] = tile_info + 2 ;
    bg_map[tile_on_screen + 1] = tile_info + 3 ;
  }
}





static void print_submission (u16 column, u16 line, const unsigned char *str)
{
  u16 *wlist_map = SCREEN_BASE_BLOCK_25_ADDR ;
  short i ;
  u16 c ;

  for (i = 0; str[i] != '\0'; i++) {
    c = wlist_char_to_tile_table[str[i]] + WLIST_SFONTS_START_INDEX ;
    wlist_map [(line * (256/8)) + column] =
      (u16) (c | (WLIST_PALETTE_SLOT << 12)) ;
    column++ ;
  }
}



static void words_wlist_full_refresh (struct game *game)
{
  short x, y, i ;

  /* On efface l'ancienne liste visuelle. */
  clear_submissions_wlist () ;
  for (i = 0; i < MAX_DISPLAYED_SUBMISSIONS; i++) {
    x = ((i < (MAX_DISPLAYED_SUBMISSIONS / 2)) ?
	 LIST1_AREA_X_START : LIST2_AREA_X_START) ;
    y = LIST1_AREA_Y_START + (i % (MAX_DISPLAYED_SUBMISSIONS / 2)) ;
    if (game->display[i] != NULL) {
      if (game->display[i]->item != NULL)
	print_submission (x, y, game->display[i]->item) ;
    }
    else break ;    /* Plus rien  afficher. */
  }
}




static void next_page (struct game *game)
{
  short i ;
  struct list_item *next_to_display ;

  next_to_display = game->display[MAX_DISPLAYED_SUBMISSIONS - 1] ;
  if (next_to_display != NULL) {
    next_to_display = next_to_display->next ;
    if (next_to_display != NULL) {
      /* Il y a vraiment besoin de tourner la page. */
      /* On dcale la page de droite vers la gauche. */
      for (i = MAX_DISPLAYED_SUBMISSIONS/2; i<MAX_DISPLAYED_SUBMISSIONS; i++) 
	game->display[i - (MAX_DISPLAYED_SUBMISSIONS / 2)] = game->display[i] ;

      /* Maintenant, on remplit la page de droite. */
      for (i=MAX_DISPLAYED_SUBMISSIONS/2; i < MAX_DISPLAYED_SUBMISSIONS; i++) {
	game->display[i] = next_to_display ;
	if (next_to_display != NULL) next_to_display = next_to_display->next ;
      }            /* Fin de for (i=MAX_DISPLAYED_SUBMISSIONS/2; */
      words_wlist_full_refresh (game) ;
    }              /* Fin du second if (next_to_display != NULL). */
    else
      popup_message (FALSE, messages[Msg_already_last_page][language]) ;
  }                /* Fin du premier if (next_to_display != NULL). */
  else
    popup_message (FALSE, messages[Msg_already_last_page][language]) ;
}




static void prev_page (struct game *game)
{
  short i ;
  struct list_item *prev_to_display ;
  
  prev_to_display = game->display[0] ;
  if (prev_to_display != NULL) {
    prev_to_display = prev_to_display->prev ;
    if (prev_to_display != NULL) {
      /* Il y a vraiment besoin de tourner la page. */
      /* On dcale la page de gauche vers la droite. */
      for (i = 0; i < MAX_DISPLAYED_SUBMISSIONS/2; i++)
	game->display[i + (MAX_DISPLAYED_SUBMISSIONS / 2)] = game->display[i] ;

      /* Maintenant, on remplit la page de droite. */
      for (i = (MAX_DISPLAYED_SUBMISSIONS/2)-1 ; i >= 0; i--) {
	game->display[i] = prev_to_display ;
	if (prev_to_display != NULL) prev_to_display = prev_to_display->prev ;
      }    /* Fin de for (i = (MAX_DISPLAYED_SUBMISSIONS/2)-1; */
      words_wlist_full_refresh (game) ;
    }              /* Fin du second if (prev_to_display != NULL). */
    else
      popup_message (FALSE, messages[Msg_already_first_page][language]) ;
  }                /* Fin du premier if (prev_to_display != NULL). */
  else
    popup_message (FALSE, messages[Msg_already_first_page][language]) ;
}





static void wlist_extend_with_submission (struct game *game)
{
  short i ;
  short x, y ;

  /* On recherche le premier slot de libre. */
  for (i = 0; i < MAX_DISPLAYED_SUBMISSIONS; i++) {
    if (game->display[i] == NULL) break ;
  }

  /* Ici, si i == MAX_DISPLAYED_SUBMISSIONS c'est qu'il n'y en n'a plus. */
  if (i < MAX_DISPLAYED_SUBMISSIONS) {
    x = ((i < (MAX_DISPLAYED_SUBMISSIONS / 2)) ?
	 LIST1_AREA_X_START : LIST2_AREA_X_START) ;
    y = LIST1_AREA_Y_START + (i % (MAX_DISPLAYED_SUBMISSIONS / 2)) ;
    game->display[i] = game->submissions_tail ;
    print_submission (x, y, game->submissions_tail->item) ;
  }
}





static void progress_bar_refresh (short pbar_y_start, short percent)
{
  u16 *wlist_map = SCREEN_BASE_BLOCK_25_ADDR ;
  short nb_ticks, nb_full_tiles, partial_tile, i ;
  u16 tile_on_screen ;

  nb_ticks = (short) ((percent * PROGRESS_BAR_LEN * 8) / 100) ;
  /* Pour viter d'afficher plus de 100 %. */
  if (nb_ticks > (PROGRESS_BAR_LEN * 8))
    nb_ticks = PROGRESS_BAR_LEN * 8 ;
  nb_full_tiles = nb_ticks / 8 ;
  partial_tile = nb_ticks % 8 ;

  tile_on_screen =
    (pbar_y_start * (256/8)) + PROGRESS_BAR_AREA_X_START ;
  /* Dessin des tiles pleines. */
  for (i = 0; i < nb_full_tiles; i++) {
    wlist_map [tile_on_screen] =
      (u16) ((PROGRESS_START_INDEX + 8) | (WLIST_PALETTE_SLOT << 12)) ;
    tile_on_screen++ ;
  }
  /* Dessin de l'ventuelle tile partielle. */
  wlist_map [tile_on_screen] =
    (u16) ((PROGRESS_START_INDEX + partial_tile) |
	   (WLIST_PALETTE_SLOT << 12)) ;
  /* Comme la barre de progrs ne diminue jamais, inutile de dessiner */
  /* la suite des blocs qui de toute faon seront transparents.       */
}





static void clear_progress_bar (short pbar_y_start)
{
  u16 *wlist_map = SCREEN_BASE_BLOCK_25_ADDR ;
  short i, tile_on_screen ;

  tile_on_screen =
    (pbar_y_start * (256/8)) + PROGRESS_BAR_AREA_X_START ;
  for (i = 0; i < PROGRESS_BAR_LEN; i++) {
    wlist_map [tile_on_screen] =
      (u16) (PROGRESS_START_INDEX | (WLIST_PALETTE_SLOT << 12)) ;
    tile_on_screen++ ;
  }
}




/* Raffrachit le compteur de mots trouvs, la barre de progrs et le score. */
static void counters_refresh (struct game *game)
{
  Letter tmp_str[8] ;
  short time_percent ;

  sprintf (tmp_str, "%3d/%3d", game->nb_found, game->nb_possibilities) ;
  print_submission
    (WORDS_COUNTER_AREA_X_START, WORDS_COUNTER_AREA_Y_START, tmp_str) ;
  progress_bar_refresh (PROGRESS_BAR_PERF_AREA_Y_START,
			game->progress_percent) ;
  time_percent = (short) ((REG_TM3CNT_L * 100) / game->available_time) ;
  progress_bar_refresh (PROGRESS_BAR_TIME_AREA_Y_START,	time_percent) ;
  sprintf (tmp_str, "%7ld", game->score) ;
  print_submission
    (SCORE_AREA_X_START, SCORE_AREA_Y_START, tmp_str) ;
}





static void process_elapsed_time (struct game *game)
{
  stop_game_timer () ;
  switch (game->game_status) {
  case Game_attempt:
    popup_message (FALSE, messages[Msg_elapsed][language]) ;
    game->game_status = Game_over ;
    break ;

  case Game_extra:
  case Game_level_complete:
    popup_message (FALSE, messages[Msg_elapsed_next_level][language]) ;
    start_game (game->score, (game->level + 1)) ;
    initial_refresh (game) ;
    break ;

  default:
    popup_message (FALSE, "process_surrender\nAssert 0.") ;
    break ;
  }
}





static void process_surrender (struct game *game)
{
  switch (game->game_status) {
  case Game_attempt:
    if (popup_message (TRUE, messages[Msg_surrender_question][language])) {
      stop_game_timer () ;
      game->game_status = Game_over ;
    }
    break ;

  case Game_extra:
  case Game_level_complete:
    if (popup_message (TRUE, messages[Msg_next_level_question][language])) {
      stop_game_timer () ;
      start_game (game->score, (game->level + 1)) ;
      initial_refresh (game) ;
    }
    break ;

  default:
    popup_message (FALSE, "process_surrender\nAssert 0.") ;
    break ;
  }
}





static void key_A_process (struct game *game,
			   enum main_loop_selection cursor_selection)
{
  switch (cursor_selection) {
  case Sel_letter_0:
  case Sel_letter_1:
  case Sel_letter_2:
  case Sel_letter_3:
  case Sel_letter_4:
  case Sel_letter_5:
  case Sel_letter_6:
    if (try_add_letter_to_submission (game, (short) cursor_selection))
      background_refresh (game) ;
    else {
      play_sound (g_wav_bad_key) ;
      shake_and_blink_screen () ;
    }
    break ;

  case Sel_shuffle:
    shuffle_letters (game) ;
    background_refresh (game) ;
    break ;

  case Sel_surrender:
      process_surrender (game) ;
    break ;

  default:
    popup_message (FALSE, "game_main_loop\nAssert 0.") ;
    break ;
  }
}





static void key_START_process (struct game *game)
{
  switch (process_user_submission (game)) {
  case Correct:
    background_refresh (game) ;
    wlist_extend_with_submission (game) ;
    counters_refresh (game) ;
    break ;
  case Too_Short:
    play_sound (g_wav_bad_key) ;
    shake_and_blink_screen () ;
    popup_message (FALSE, messages[Msg_3_letters_min][language]) ;
    break ;
  case Already_Sumbitted:
    play_sound (g_wav_inexist) ;
    popup_message (FALSE, messages[Msg_already_proposed][language]) ;
    break ;
  case Non_Existing:
    play_sound (g_wav_inexist) ;
    popup_message (FALSE, messages[Msg_unknown_word][language]) ;
    break ;
  case Level_successful:
    /* On vient juste d'atteindre la perf minimale */
    /* pour passer au niveau suivant.              */
    background_refresh (game) ;
    wlist_extend_with_submission (game) ;
    counters_refresh (game) ;
    play_sound (g_wav_next_level) ;
    popup_message (FALSE, messages[Msg_min_perf][language]) ;
    break ;
  case Level_fully_complete:
    play_sound (g_wav_next_level) ;
    popup_message (FALSE, messages[Msg_all_possible_words][language]) ;
    start_game (game->score, (game->level + 1)) ;
    initial_refresh (game) ;
    break ;
  default:
    popup_message (FALSE, "game_main_loop\nAssert 1.") ;
    break ;
  }
}





void game_main_loop (struct game *game)
{
  enum main_loop_selection cursor_selection, old_cursor_selection ;
  short time_percent ;
  u16 five_last_seconds_counter ;

  five_last_seconds_counter = game->available_time - 6 ;
  cursor_selection = Sel_letter_0 ;
  /* Positionnement initial du sprite de curseur. */
  pointer_sprite.attribute1 = pointer_sprite.attribute1 & 0xFE00 ;
  pointer_sprite.attribute1 =
    pointer_sprite.attribute1 | SEL_LETTER_0_X_POS ;
  pointer_sprite.attribute0 = pointer_sprite.attribute0 & 0xFF00 ;
  pointer_sprite.attribute0 = pointer_sprite.attribute0 | SEL_LETTERS_Y_POS  ;
  while (REG_VCOUNT < 160) ;
  memcpy (sprite_video_memory, &pointer_sprite, sizeof (struct sprite)) ;

  while (game->game_status != Game_over) {
    /* Gestion du temps. */
    if (REG_TM3CNT_L >= game->available_time) process_elapsed_time (game) ;
    /* Gestion de l'alarme. */
    if (REG_TM3CNT_L > five_last_seconds_counter) {
      five_last_seconds_counter = REG_TM3CNT_L ;
      play_sound (g_wav_timer) ;
    }
    time_percent = (short) ((REG_TM3CNT_L * 100) / game->available_time) ;
    progress_bar_refresh (PROGRESS_BAR_TIME_AREA_Y_START, time_percent) ;

    old_cursor_selection = cursor_selection ;
    KEY_POOL ;

    if (KEY_HIT (BUTTON_A)) {
      key_A_process (game, cursor_selection) ;
      goto redraw ;
    }

    if (KEY_HIT (BUTTON_B)) {
      if (try_remove_letter_from_submission (game))
	background_refresh (game) ;
      else {
	play_sound (g_wav_bad_key) ;
	shake_and_blink_screen () ;
      }
      goto redraw ;
    }

    if (KEY_HIT (BUTTON_RIGHT)) {
      next_page (game) ;
      goto redraw ;
    }

    if (KEY_HIT (BUTTON_LEFT)) {
      prev_page (game) ;
      goto redraw ;
    }

    if (KEY_HIT (KEYPAD_UP)) {
      switch (cursor_selection) {
      case Sel_shuffle : cursor_selection = Sel_letter_6 ; break ;
      case Sel_surrender : cursor_selection = Sel_shuffle ; break ;
      default: break ;
      }
      goto redraw ;
    }
    
    if (KEY_HIT (KEYPAD_DOWN)) {
      switch (cursor_selection) {
      case Sel_shuffle : cursor_selection = Sel_surrender ; break ;
      case Sel_surrender : cursor_selection = Sel_letter_6 ; break ;
      default: break ;
      }
      goto redraw ;
    }

    if (KEY_HIT (KEYPAD_LEFT)) {
      /* Dplacement vers la gauche. */
      if (cursor_selection == Sel_letter_0) cursor_selection = Sel_surrender ;
      else cursor_selection-- ;
      goto redraw ;
    }

    if (KEY_HIT (KEYPAD_RIGHT)) {
      /* Dplacement vers la droite. */
      if (cursor_selection == Sel_surrender) cursor_selection = Sel_letter_0 ;
      else cursor_selection++ ;
      goto redraw ;
    }

    /* Validation de la suggestion. */
    if (KEY_HIT (BUTTON_START)) key_START_process (game) ;

    if (KEY_HIT (BUTTON_SELECT)) {
      /* Appel de la pause. */
      popup_message (FALSE, messages[Msg_pause_A_to_continue][language]) ;
      goto redraw ;
    }

  redraw:
    /* On change la position du curseur seulement s'il a boug. */
    if (cursor_selection != old_cursor_selection) {
      u16 x = SEL_LETTER_0_X_POS ;
      u16 y = SEL_LETTERS_Y_POS ;
      switch (cursor_selection) {
      case Sel_letter_0:
	x = SEL_LETTER_0_X_POS ;
	y = SEL_LETTERS_Y_POS ;
	break ;
      case Sel_letter_1:
	x = SEL_LETTER_1_X_POS ;
	y = SEL_LETTERS_Y_POS ;
	break ;
      case Sel_letter_2:
	x = SEL_LETTER_2_X_POS ;
	y = SEL_LETTERS_Y_POS ;
	break ;
      case Sel_letter_3:
	x = SEL_LETTER_3_X_POS ;
	y = SEL_LETTERS_Y_POS ;
	break ;
      case Sel_letter_4:
	x = SEL_LETTER_4_X_POS ;
	y = SEL_LETTERS_Y_POS ;
	break ;
      case Sel_letter_5:
	x = SEL_LETTER_5_X_POS ;
	y = SEL_LETTERS_Y_POS ;
	break ;
      case Sel_letter_6:
	x = SEL_LETTER_6_X_POS ;
	y = SEL_LETTERS_Y_POS ;
	break ;
      case Sel_shuffle:
	x = SEL_SHUFFLE_X_POS ;
	y = SEL_SHUFFLE_Y_POS ;
	break ;
      case Sel_surrender:
	x = SEL_SURRENDER_X_POS ;
	y = SEL_SURRENDER_Y_POS ;
	break ;
      default:
	popup_message (FALSE, "game_main_loop\nAssert 1.") ;
	break ;
      }

      pointer_sprite.attribute1 = pointer_sprite.attribute1 & 0xFE00 ;
      pointer_sprite.attribute1 = pointer_sprite.attribute1 | x ;
      pointer_sprite.attribute0 = pointer_sprite.attribute0 & 0xFF00 ;
      pointer_sprite.attribute0 = pointer_sprite.attribute0 | y  ;
      /* Transfert des infos de sprite dans la mmoire vido-sprite. */
      while (REG_VCOUNT < 160) ;
      memcpy (sprite_video_memory, &pointer_sprite, sizeof (struct sprite)) ;
    }      /* Fin de if (cursor_selection != old_cursor_selection) */
  }        /* Fin de for (;;) */
}





void initial_refresh (struct game* game)
{
  /* On active le curseur. */
  set_screen_mode (REG_DISPCNT | OBJ_ENABLE) ;
  clear_submissions_wlist () ;
  clear_progress_bar (PROGRESS_BAR_PERF_AREA_Y_START) ;
  clear_progress_bar (PROGRESS_BAR_TIME_AREA_Y_START) ;
  background_refresh (game) ;
  counters_refresh (game) ;
}




static void show_solutions (struct game *game)
{
  short i ;
  struct list_item *to_display ;

  to_display = game->possibilities ;
  for (i = 0; i < MAX_DISPLAYED_SUBMISSIONS; i++) {
    game->display[i] = to_display ;
    if (to_display != NULL) to_display = to_display->next ;
  }
  words_wlist_full_refresh (game) ;
}




/* Affiche les scores des highscores dans le layer 2. */
void refresh_fame_display ()
{
  unsigned char tmp_str[8] ;

  clear_submissions_wlist () ;
  clear_progress_bar (PROGRESS_BAR_PERF_AREA_Y_START) ;
  clear_progress_bar (PROGRESS_BAR_TIME_AREA_Y_START) ;

  sprintf (tmp_str, "%7lu", global_hall_of_fame[0]) ;
  print_submission (SLOT1_X, SLOT1_Y, tmp_str) ;
  sprintf (tmp_str, "%7lu", global_hall_of_fame[1]) ;
  print_submission (SLOT2_X, SLOT2_Y, tmp_str) ;
  sprintf (tmp_str, "%7lu", global_hall_of_fame[2]) ;
  print_submission (SLOT3_X, SLOT3_Y, tmp_str) ;
  sprintf (tmp_str, "%7lu", global_hall_of_fame[3]) ;
  print_submission (SLOT4_X, SLOT4_Y, tmp_str) ;
  sprintf (tmp_str, "%7lu", global_hall_of_fame[4]) ;
  print_submission (SLOT5_X, SLOT5_Y, tmp_str) ;
}




/* A pour but de laisser le joueur regarder les possibilit aprs que le */
/* jeu soit termin et attend qu'il veuille en recommencer un nouveau.   */
void game_ended_loop (struct game *game)
{
  /* On dsactive le curseur. */
  set_screen_mode (REG_DISPCNT & (~OBJ_ENABLE)) ;
  init_game_over () ;
  update_high_scores (game) ;
  init_fame () ;
  reload_game_background_graphics () ;
  show_solutions (game) ;
  for (;;) {
    KEY_POOL ;

    if (KEY_HIT (BUTTON_LEFT)) prev_page (game) ;

    if (KEY_HIT (BUTTON_RIGHT)) next_page (game) ;

    if (KEY_HIT (BUTTON_A)) break ;
  }
}
