/*
  *** Blabla GBA ***
 Franois PESSAUX Dcember 2004
 This program is freeware.
 Use its sources as you whant provided that you leave this copyright unchanged
 and that you drop a small word telling that I'm the author of these sources.
 Thank's a lot.
   --Francois
*/

#include <string.h>
#include <gba.h>
#include <screen_mode.h>
#include "basic_types_defs.h"
#include "game.h"
#include "init.h"
#include "key_input.h"
#include "intro.h"
#include "intro_data.h"
#include "intro_pointer_data.h"



enum language language = Lang_French ;



static void handle_intro_input ()
{
  enum language language_selection, old_language_selection ;

  pointer_sprite.attribute1 = pointer_sprite.attribute1 & 0xFE00 ;
  pointer_sprite.attribute1 =
    pointer_sprite.attribute1 | CURSOR_INTRO_X_FRENCH ;
  /* Efface l'ancienne ordonne. */
  pointer_sprite.attribute0 = pointer_sprite.attribute0 & 0xFF00 ;
  /* Mme genre de correction pour l'ordonne. */
  pointer_sprite.attribute0 = pointer_sprite.attribute0 | CURSOR_INTRO_Y ;
  /* Transfert des infos de sprite dans la mmoire vido-sprite. */
  while (REG_VCOUNT < 160) ;
  memcpy (sprite_video_memory, &pointer_sprite, sizeof (struct sprite)) ;

  language_selection = Lang_French ;
  /* On attend une pression de touche Start pour continuer. */
  for(;;) {
    old_language_selection = language_selection ;

    KEY_POOL ;
    if ((KEY_HIT (BUTTON_A)) || (KEY_HIT(BUTTON_START)) ||
	(KEY_HIT (BUTTON_SELECT))) {
      language = language_selection ;
      return ;
      }

    if (KEY_HIT (KEYPAD_LEFT)) {
      /* Dplacement vers la gauche. */
      if (language_selection == Lang_French)
	language_selection = Lang_English ;
      else
	language_selection = Lang_French ;
    }
    
    if (KEY_HIT (KEYPAD_RIGHT)) {
      /* Dplacement vers la droite. */
      language_selection = (language_selection + 1) % 2 ;
    }

    /* On change la position du curseur seulement s'il a boug. */
    if (language_selection != old_language_selection) {
      u16 x = CURSOR_INTRO_X_FRENCH ;
      switch (language_selection) {
      case Lang_French: x = CURSOR_INTRO_X_FRENCH ; break ;
      case Lang_English: x = CURSOR_INTRO_X_ENGLISH ; break ;
      }
      pointer_sprite.attribute1 = pointer_sprite.attribute1 & 0xFE00 ;
      pointer_sprite.attribute1 = pointer_sprite.attribute1 | x ;
      /* Transfert des infos de sprite dans la mmoire vido-sprite. */
      while (REG_VCOUNT < 160) ;
      memcpy (sprite_video_memory, &pointer_sprite, sizeof (struct sprite)) ;
    }
  }
}





void init_intro ()
{
  u16 i, j, k ;
  u16 *bg_pal = BACKGROUND_PALETTE_ADDR ;
  u16 *bg_tiles = CHAR_BASE_BLOCK_0_ADDR ;
  u16 *bg_map = SCREEN_BASE_BLOCK_24_ADDR ;
  u16 *ptr_tiles = SPRITES_CHAR_BASE_BLOCK_ADR ;
  u16 *sprite_palette = SPRITES_PALETTE_ADR ;
  struct sprite *sprite_video_memory = SPRITES_INFO_MEMORY_ADR ;

    /* Fond. */
  set_background0
    (BACKGROUND_PRIORITY_3 |            /* Priorit la plus faible. */
     BACKGROUND_CHAR_BASE_BLOCK_0 |     /* Tiles */
     BACKGROUND_COLOR_256 |             /* 256 couleurs. */
     BACKGROUND_SCREEN_BASE_BLOCK_24 |  /* Map. */
     TEXT_BACKGROUND_256x256) ;         /* 256 x 256 pixels. */

  /* Palette du fond. */
  for (i = 0; i < INTRO_PALETTE_SIZE; i++)
    bg_pal[i] = intro_palette[i] ;

  /* Chargement des tiles du fond d'cran en mmoire. */
  k = 0 ;
  for (i = 0; i < INTRO_NB_TILES * (64 / 2); i++) {
    bg_tiles[k] =
      (u16) ((intro_tiles[(i * 2) + 1]<< 8) |
	     (intro_tiles[(i * 2)])) ;
    k++ ;
  }

  /* Chargement de la map de fond. */
  k = 0 ;
  for (j = 0; j < 20; j++) {          /* Boucle verticale. */
    for (i = 0; i < 30; i++) {        /* Boucle honrizontale. */
      bg_map[(j * (256 / 8)) + i] = (u16) intro_map[k] ;
      k++ ;
    }
  }

  /* Chargement des tiles des sprites. */
  for (i = 0; i < INTRO_POINTER_NB_TILES * (32 / 2) ; i++) 
    ptr_tiles[i] =
      (u16) ((intro_pointer_tiles[(i * 2) + 1]<< 8) |
	     (intro_pointer_tiles[(i * 2)])) ;

  /* Chargement de la palette des sprites. */
  for (i = 0; i < INTRO_POINTER_PALETTE_SIZE ; i++)
    sprite_palette[i] = intro_pointer_palette[i] ;

  /* Cache tous les sprites en dehors de l'cran pour que */
  /* ceux que l'on n'utilisera pas ne se voient pas.      */
  for (i = 0; i < 128; i++) {
    sprite_video_memory[i].attribute0 = 160 ; /* y vers > 159. */
    sprite_video_memory[i].attribute1 = 240 ; /* x vers  > 239. */
  }

  /* Initialisation du sprite du curseur. */
  pointer_sprite.attribute0 = SPRITE_16_COLOR | SPRITE_SQUARE ;
  pointer_sprite.attribute1 = SPRITE_SIZE_16 ;
  pointer_sprite.attribute2 = INTRO_POINTER_SPRITE_TILE_INDEX ;

  REG_BG0HOFS = 0 ; REG_BG0VOFS = 0 ;
  set_screen_mode (MODE_0 | BG0_ENABLE | OBJ_MAP_1D | OBJ_ENABLE) ;

  handle_intro_input () ;
}
